using System.Collections.Generic;
using communicate_rental_backend_dev.Models.DataReceiving;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using communicate_rental_backend_dev.Mappers;
using System.Linq;

namespace communicate_rental_backend_dev.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class FeedbackController : Controller
    {
        private readonly IFeedbackRepository feedbackRepository;
        private readonly IBlobFeedbackRepository blobFeedbackRepository;
        private readonly IFeedbackMapper feedbackMapper;

        public FeedbackController(IFeedbackRepository feedbackRepository,
        IFeedbackMapper feedbackMapper,
        IBlobFeedbackRepository blobFeedbackRepository)
        {
            this.feedbackRepository = feedbackRepository;
            this.feedbackMapper = feedbackMapper;
            this.blobFeedbackRepository = blobFeedbackRepository;
        }

        /// <summary>
        /// Adds feedback to table storage
        /// </summary>
        /// <param name="feedbackDRO"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<IActionResult> Add([FromBody]FeedbackDRO feedbackDRO)
        {
            var feedback = feedbackMapper.Map(feedbackDRO);
            await feedbackRepository.Add(feedback);

            return Created(feedback.FeedbackAbout, true);
        }

        /// <summary>
        /// Get all feedback
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<FeedbackDRO>>> GetAll()
        {
            var feedback = await feedbackRepository.GetAll();
            if (feedback == null || !feedback.Any())
            {
                return NotFound();
            }

            var list = new List<FeedbackDRO>();
            foreach (var item in feedback)
            {
                list.Add(feedbackMapper.Map(item));
            }
            return list;
        }

        /// <summary>
        /// Get feedback by id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("{id}")]
        public async Task<ActionResult<FeedbackDRO>> GetById([FromRoute]string id)
        {
            var feedback = await feedbackRepository.Get(id);
            if (feedback == null)
            {
                return NotFound();
            }
            return feedbackMapper.Map(feedback);
        }

        /// <summary>
        /// Moves feedback from table storage to blob
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Move/{id}")]
        public async Task<ActionResult> MoveToBlob(string id)
        {
            var feedback = await feedbackRepository.Get(id);
            if (id == null)
            {
                return NotFound();
            }
            try
            {
                await blobFeedbackRepository.UploadAsync(feedback);
                await feedbackRepository.Delete(feedback.Id);
            }
            catch (System.Exception)
            {
                
                return StatusCode(500, "Kunne ikke flytte tilbakemelding");
            }
            return Accepted("Tilbakemeldingen ble flyttet");
        }

    }
}
