using System;
using communicate_rental_backend_dev.BusinessLogic.Services;
using Microsoft.AspNetCore.Mvc;
namespace communicate_rental_backend_dev.Controllers
{
    [Route("[controller]")]
    public class FunctionAppController : Controller
    {
        private readonly IFunctionAppService functionAppService;

        public FunctionAppController(IFunctionAppService functionAppService)
        {
            this.functionAppService = functionAppService; 
        }

        /// <summary>
        /// Triggered by function app, triggers once every day
        /// - Updates user points
        /// - Allocates rentals and sends reminders
        /// - Updates holidays
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [Route("Trigger")]
        public void Scheduler()
        {
            var today = DateTime.Today;

            functionAppService.UpdateUserPoints(today);

            functionAppService.AllocateRentals(today);

            functionAppService.UpdateAllocationPeriods(today);

            functionAppService.MoveBookingsToBlobOrSendChecklistReminder(today);

            functionAppService.SendListOfBookingsForMonth(today);


            // If 1. jan - make sure the holidays for the next year is uploaded to db
            //  remove holidays for last year
            if (today.Date == new DateTime(today.Year, 1, 1).ToUniversalTime().Date)
            {
                functionAppService.UpdateHolidays(today);
            }
        }
    }
}
