using System;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace communicate_rental_backend_dev.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class HolidayController : Controller
    {
        private readonly IHolidayRepository holidayRepository;
        private readonly IHolidayMapper holidayMapper;

        public HolidayController(IHolidayRepository holidayRepository,
        IHolidayMapper holidayMapper)
        {
            this.holidayRepository = holidayRepository;
            this.holidayMapper = holidayMapper;
        }

        /// <summary>
        /// Get all holidays in database 
        /// </summary>
        /// <returns>Ienumerable list of holidayDTOs</returns>
        [HttpGet]
        public async Task<IEnumerable<HolidayDTO>> GetHolidays()
        {
            var holidays = await holidayRepository.GetAll();

            if (holidays == null) 
            {
                return null;
            }

            var holidayDTO = new List<HolidayDTO>(); 
            
            foreach (var day in holidays)
            {
                holidayDTO.Add(holidayMapper.Map(day));
            }

            return holidayDTO;
        }

        /// <summary>
        /// Adds holidays passed from frontend if they are not conflicting with existing
        /// </summary>
        /// <param name="holidayDRO">Data Recieving object of type holiday</param>
        /// <returns>statuscode 201 will always be returned, but exception is console logged if operation failed</returns>
        [HttpPost]
        [Route("add")]
        public async Task<IActionResult> Add([FromForm]HolidayDRO holidayDRO) 
        {
            var holidays = holidayMapper.Map(holidayDRO);
            var existingHolidays = await holidayRepository.GetAll();
            int counter = 0;

            foreach (var day in holidays)
            {
                bool matchingDay = false;
                if (existingHolidays != null) {
                    foreach (var d in existingHolidays)
                    {
                        if (day.DateOfHoliday == d.DateOfHoliday) 
                        { 
                            Console.WriteLine(new ArgumentOutOfRangeException("Conflicting holiday, " + day.DateOfHoliday.Date.ToShortDateString() + " already exists."));
                            matchingDay = true;
                            break;
                        }
                    }
                }
                if (matchingDay == true) { continue; }
                await holidayRepository.Add(day);
                counter ++; 
            }
            if (counter > 0) 
            {
                return StatusCode(201, counter + " holiday(s) was added!");
            }
            return StatusCode(500, "All holidays already exists in database.");
        }
         
        /// <summary>
        /// Deletes holiday by id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{id}")]
        public async Task<IActionResult> Delete([FromRoute]string id) 
        {
            await holidayRepository.Delete(id);

            return StatusCode(200, id + " was deleted");
        }
    }
} 