using System;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;

namespace communicate_rental_backend_dev.Mappers
{

    public class AllocationPeriodMapper : IAllocationPeriodMapper 
    {
        // Maps AllocationPeriod from DRO to RentalObject DB property (serialize Id)
        public AllocationPeriod Map(AllocationPeriodDTO allocationPeriodDRO, string rentalObject) 
        {
            if (allocationPeriodDRO == null) 
            {
                return null;
            }
            return new AllocationPeriod {
                Id = Guid.NewGuid().ToString(),
                AllocationStart = allocationPeriodDRO.AllocationStart.ToUniversalTime(),
                AllocationEnd = allocationPeriodDRO.AllocationEnd.ToUniversalTime(),
                ApplicationDeadline = allocationPeriodDRO.ApplicationDeadline.ToUniversalTime(),
                RentalObject = rentalObject,
            };
        }

        /// <summary>
        /// Maps Allocationperiod DB model to DTO object
        /// </summary>
        /// <param name="allocationPeriod">Allocationperiod stored in db, passed from controller</param>
        /// <returns>Allocationperiod DTO object</returns>
        public AllocationPeriodDTO Map(AllocationPeriod allocationPeriod) 
        {
            if (allocationPeriod == null) 
            {
                return null;
            }
            return new AllocationPeriodDTO {
                AllocationStart = allocationPeriod.AllocationStart.ToLocalTime(),
                AllocationEnd = allocationPeriod.AllocationEnd.ToLocalTime(),
                ApplicationDeadline = allocationPeriod.ApplicationDeadline.ToLocalTime(),
            };
        }
    }
}
