using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;
using System;

namespace communicate_rental_backend_dev.Mappers
{
    public class BookingMapper : IBookingMapper
    {
        
        /// <summary>
        /// Generates id of booking
        /// </summary>
        /// <param name="booking">bookingDRO</param>
        /// <returns>new booking</returns>
        public Booking Map(BookingDRO booking) 
        {
            return new Booking {
                Id = Guid.NewGuid().ToString(), 
                UserId = booking.UserId, 
                EnglishTitle = booking.EnglishTitle, 
                StartDate = booking.StartDate.ToUniversalTime(), 
                EndDate = booking.EndDate.ToUniversalTime(), 
                IsConfirmed = false, 
                IsInHoliday = false, 
                Title = booking.Title,
                AdminComment = booking.AdminComment
            };
        }

         /// <summary>
        /// Generates id of booking
        /// </summary>
        /// <param name="booking">bookingDRO</param>
        /// <returns>new booking</returns>
        public Booking MapUpdate(BookingDRO booking) 
        {
            return new Booking {
                Id = booking.Id, 
                UserId = booking.UserId, 
                EnglishTitle = booking.EnglishTitle, 
                StartDate = booking.StartDate.ToUniversalTime(), 
                EndDate = booking.EndDate.ToUniversalTime(), 
                IsConfirmed = false, 
                IsInHoliday = false, 
                Title = booking.Title,
                AdminComment = booking.AdminComment
            };
        }

        /// <summary>
        /// Generates a simplified DTO from DB model
        /// </summary>
        /// <param name="booking">DB model</param>
        /// <returns>bookingDTO</returns>
        public BookingDTO Map(Booking booking) 
        {
            return new BookingDTO 
            {
                Id = booking.Id,
                EnglishTitle = booking.EnglishTitle, 
                IsConfirmed = booking.IsConfirmed, 
                Title = booking.Title,
                StartDate = booking.StartDate.ToLocalTime(), 
                EndDate = booking.EndDate.ToLocalTime(), 
                AdminComment = booking.AdminComment
            };
        }

        /// <summary>
        /// Generates a DTO from DB model and user's name
        /// </summary>
        /// <param name="booking">DB model</param>
        /// <param name="name">User's name</param>
        /// <returns></returns>
        public BookingDTO Map(Booking booking, string name) 
        {
            return new BookingDTO{
                Id = booking.Id,
                EnglishTitle = booking.EnglishTitle, 
                UserName = name,
                IsConfirmed = booking.IsConfirmed, 
                Title = booking.Title,
                StartDate = booking.StartDate.ToLocalTime(), 
                EndDate = booking.EndDate.ToLocalTime(), 
                AdminComment = booking.AdminComment
            };
        }
    }
}