using System;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Models.Common;
using System.Text.Json;
using System.Collections.Generic;

namespace communicate_rental_backend_dev.Mappers
{
    public class ChecklistMapper : IChecklistMapper 
    {
        private readonly IMapperUtility mapperUtility;

        public ChecklistMapper(IMapperUtility mapperUtility)
        {
            this.mapperUtility = mapperUtility;
        }

        // Maps checklist from DRO to DB model
        public Checklist Map(ChecklistDRO checklistDRO) 
        {
            if (checklistDRO == null) 
            {
                return null;
            }

            // Set encoding type to allow all unicode characters to not break Norwegian
            var options = mapperUtility.GetEncoderSettings();

            return new Checklist 
            {
                Id = Guid.NewGuid().ToString(),
                UserId = checklistDRO.UserId,
                BookingId = checklistDRO.BookingId,
                LocationTitle = checklistDRO.LocationTitle,
                StartDate = checklistDRO.StartDate.ToUniversalTime(),
                EndDate = checklistDRO.EndDate.ToUniversalTime(),
                FilledChecklist = JsonSerializer.Serialize(checklistDRO.ChecklistItems, options),
                Comment = checklistDRO.Comment
            };
        }

        /// <summary>
        /// desierialize checklist 
        /// </summary>
        /// <param name="checklist"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public ChecklistDTO Map(Checklist checklist, string name) 
        {
            if (checklist == null)
            {
                return null;
            }
            else 
            {
                return new ChecklistDTO 
                {
                    ChecklistId = checklist.Id,
                    UserName = name,
                    LocationTitle = checklist.LocationTitle,
                    StartDate = checklist.StartDate.ToLocalTime(),
                    EndDate = checklist.EndDate.ToLocalTime(),
                    FilledChecklist = mapperUtility.DeSerializeToArray<FilledChecklist>(checklist.FilledChecklist),
                    Comment = checklist.Comment
                };
            }
        }

        /// <summary>
        /// maps string-array to checklistDTO
        /// </summary>
        /// <param name="emptyChecklist"></param>
        /// <returns></returns>
        public ChecklistDTO Map(string[] emptyChecklist) 
        {
            if (emptyChecklist == null)
            {
                return null;
            }
            else 
            {
                var list = new List<FilledChecklist>();
                foreach (var item in emptyChecklist)
                {
                    var checklistItem = new FilledChecklist 
                    {
                        Description = item,
                        Completed = false
                    };
                    list.Add(checklistItem);
                }

                return new ChecklistDTO 
                {
                    FilledChecklist = list.ToArray()
                };

            }
        }

    }
}