using System;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.Database;
using System.Collections.Generic;
using communicate_rental_backend_dev.Models.DataTransfer;

namespace communicate_rental_backend_dev.Mappers
{
    public class HolidayMapper : IHolidayMapper 
    {
        // Maps Holiday from DRO to DB model
        public List<Holiday> Map(HolidayDRO holidayDRO) 
        {
            if (holidayDRO == null) 
            {
                return null;
            }
            var holidays = new List<Holiday>();

            for (DateTime date = holidayDRO.StartDate; date<=holidayDRO.EndDate; date = date.AddDays(1))
            {
                var holiday = new Holiday {
                    Id = Guid.NewGuid().ToString(),
                    DateOfHoliday = date.ToUniversalTime()
                };
                holidays.Add(holiday);
            }
            return holidays;
        }
        /// <summary>
        /// Maps a date to holiday object
        /// </summary>
        /// <param name="date">date of holiday to be created</param>
        /// <returns>Holiday object</returns>
        public Holiday Map(DateTime date)
        {
            return  new Holiday {
                    Id = Guid.NewGuid().ToString(),
                    DateOfHoliday = date.ToUniversalTime()
                };
        }

        /// <summary>
        /// Maps from database model to DTO
        /// </summary>
        /// <param name="holiday"></param>
        /// <returns>DTO</returns>
        public HolidayDTO Map(Holiday holiday)
        {
            if (holiday == null)
            {
                return null;
            }
            return new HolidayDTO {
                Date = holiday.DateOfHoliday.ToLocalTime()
            };
        }
    }
}
