using System.Text.Json;
using System.Text.Encodings.Web;
using System.Text.Unicode;
using System.Collections.Generic;

namespace communicate_rental_backend_dev.Mappers
{
    public class MapperUtility : IMapperUtility 
    {
        /// <summary>
        /// Sets encoder settings to not break on norwegian
        /// </summary>
        public JsonSerializerOptions GetEncoderSettings()
        {
            var encoderSettings = new TextEncoderSettings();
            encoderSettings.AllowRange(UnicodeRanges.All);
            var options = new JsonSerializerOptions
            { Encoder = JavaScriptEncoder.Create(encoderSettings) };
            return options;
        }

        /// <summary>
        /// DeSerialize json text to array of object T
        /// </summary>
        /// <param name="input">json string</param>
        /// <typeparam name="T">object to be serialized</typeparam>
        /// <returns>deserialized object</returns>
        public T[] DeSerializeToArray<T>(string input) {
            // Create empty array of sent type if no content
            if (string.IsNullOrEmpty(input)) {
                return new T[0];
            }

            // Else return deserialised json -> filled array of sent type
            return JsonSerializer.Deserialize<T[]>(input);
        }

        public string ToJson<T>(IEnumerable<T> collection)
        {
            if (collection == null)
            {
                return null;
            }
            return JsonSerializer.Serialize(collection, GetEncoderSettings());
        }
    }
}