using System;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Models.Common;

namespace communicate_rental_backend_dev.Mappers
{
    public class UserMapper : IUserMapper 
    {
        /// <summary>
        /// Sets hireddate in year 2000 - NB! FunctionAppService use this year
        /// </summary>
        /// <param name="userDRO"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public User Map(UserDRO userDRO, string id) 
        {
            if (userDRO == null) 
            {
                return null;
            }
            
            return new User 
            {
                UserId = id,
                Points = userDRO.Points,
                HiredDate = new DateTime(2000, userDRO.HiredDate.Month, userDRO.HiredDate.Day).ToUniversalTime(),
                isAdmin = userDRO.isAdmin
            };
        }
        
        // Maps user from Db to DTO model
        public UserDTO Map(User user, GraphUser graphUser)
        {
            if (user == null || graphUser == null)
            {
                return null;
            }

            return new UserDTO {
                UserId = user.UserId,
                UserName = graphUser.displayName,
                Email = string.IsNullOrEmpty(graphUser.mail) ? graphUser.userPrincipalName : graphUser.mail,
                Points = user.Points,
                HiredDate = user.HiredDate.ToLocalTime(),
                IsAdmin = user.isAdmin
            };
        }
    }
}