using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Models.Database;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

// Sends shit to access database -> tableStorageClient
namespace communicate_rental_backend_dev
{
    public class AllocationPeriodRepository : IAllocationPeriodRepository
    {
        private readonly ITableStorageClient<AllocationPeriod> client;
        private readonly IRentalObjectRepository rentalObjectRepository;

        public AllocationPeriodRepository(ITableStorageClient<AllocationPeriod> client,
        IRentalObjectRepository rentalObjectRepository)
        {
            this.client = client;
            this.rentalObjectRepository = rentalObjectRepository;
        }

        /// <summary>
        /// Queries the allocationPeriod with deadline between the searching date +- 1 day for rentalObject
        /// -> workaraound
        /// </summary>
        /// <param name="date">applicationDeadline</param>
        /// <param name="englishTitle">id of rentalObject</param>
        /// <returns>allocationPeriod</returns>
        public async Task <IEnumerable<AllocationPeriod>> GetByDeadline(DateTime date, string englishTitle)
        {
            return await client.GetByPredicate(
                x => x.ApplicationDeadline >= date.AddDays(-1) &&
                x.ApplicationDeadline <= date.AddDays(1) &&
                x.RentalObject == englishTitle);
        }

        public async Task <IEnumerable<AllocationPeriod>> GetCurrent(DateTime today, string englishTitle)
        {
            return await client.GetByPredicate(
                x => x.AllocationStart <= today &&
                x.AllocationEnd >= today);
        }

        /// <summary>
        /// Get all allocationPeriods that has enddate earlier than today
        /// </summary>
        /// <param name="today"></param>
        /// <returns></returns>
        public async Task <IEnumerable<AllocationPeriod>> GetByPredicate(DateTime today)
        {
            return await client.GetByPredicate(
                x => x.AllocationEnd.Date < today.Date);
        }

        public async Task<AllocationPeriod> GetById(string id)
        {
            return await client.GetById(id, id);
        }

        public async Task<IEnumerable<AllocationPeriod>> GetAll()
        {
            return await client.GetAll();
        }

        public async Task<IEnumerable<AllocationPeriod>> GetByRentalObject(string englishTitle)
        {
            return await client.GetByPredicate(x => x.RentalObject == englishTitle);
        }

        // Adds AllocationPeriod passed from frontend
        public async Task Add(AllocationPeriod allocationPeriod)
        {
            await client.Upsert(allocationPeriod, allocationPeriod.Id, allocationPeriod.Id);
        }

        public async Task Delete(string id)
        {
            await client.Delete(id, id);
        }

        public AllocationPeriod AddYear(AllocationPeriod period, int year)
        {
            period.AllocationStart = new DateTime(period.AllocationStart.Year + year, period.AllocationStart.Month, period.AllocationStart.Day).ToUniversalTime();
            period.AllocationEnd = new DateTime(period.AllocationEnd.Year + year, period.AllocationEnd.Month, period.AllocationEnd.Day).ToUniversalTime();
            period.ApplicationDeadline = new DateTime(period.ApplicationDeadline.Year + year, period.ApplicationDeadline.Month, period.ApplicationDeadline.Day).ToUniversalTime();
            return period;
        }
    }
}