using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;
using System.Collections.Generic;
using System.Text.Json;
using System.Threading.Tasks;


// Sends stuff to access database via tableStorageClient
namespace communicate_rental_backend_dev
{
    public class RentalObjectRepository : IRentalObjectRepository
    {
        private readonly ITableStorageClient<RentalObject> client;
        private readonly IMapperUtility mapperUtility;

        public RentalObjectRepository(ITableStorageClient<RentalObject> client,
        IMapperUtility mapperUtility)
        {
            this.client = client;
            this.mapperUtility = mapperUtility;
        }

        // Get all IDs and titles
        public async Task<IEnumerable<RentalObjectID>> GetIds() 
        {
            var results = await client.GetAll();

            var Ids = new List<RentalObjectID>();
            foreach (var result in results)
            {
                Ids.Add(new RentalObjectID {
                    EnglishTitle = result.EnglishTitle,
                    Title = result.Title
                });
            }

            return Ids;
        }

        // Get by ID
        public async Task<RentalObject> GetById(string id)
        {
            return await client.GetById(id, id);
        }

        // Get all
        public async Task<IEnumerable<RentalObject>> GetAll() 
        {
            return await client.GetAll();
        }

        public async Task<string[]> GetChecklist(string englishTitle)
        {
            RentalObject rentalObject = await client.GetById(englishTitle, englishTitle); 
            var checklistContent = JsonSerializer.Deserialize<string[]>(rentalObject.CheckList); 

            return checklistContent; 
        }

        public async Task<string> GetRentalObjectTitle(string englishTitle)
        {
            RentalObject rentalObject = await GetById(englishTitle);
            return rentalObject.Title;
        }

        public async Task Add(RentalObject rentalObject) 
        {
            await client.Upsert(rentalObject, rentalObject.EnglishTitle, rentalObject.EnglishTitle); 
        }

        public async Task Delete(string id) 
        {
            await client.Delete(id, id);
        }

        public async Task<string> GetPeriodPrice(string englishTitle)
        {
            RentalObject rentalObject = await GetById(englishTitle);
            return rentalObject.PeriodPrices;
        }
    }
}