using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using MimeKit;
using MailKit;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using unit_tests.Utilities;
using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Models.Common;

namespace unit_tests
{
    [TestClass]
    public class BookingRepositoryTests
    {
        private Mock<ITableStorageClient<Booking>> clientMock;
        private Mock<IHolidayRepository> holidayRepositoryMock;
        private BookingRepository repository;
        private Fixture fixture;


        [TestInitialize]
        public void TestInit()
        {
            // Set up mock dependencies
            clientMock = new Mock<ITableStorageClient<Booking>>();
            holidayRepositoryMock = new Mock<IHolidayRepository>();
            repository = new BookingRepository
            (
                // Add .Object to make them correct type for class/constructor
                clientMock.Object,
                holidayRepositoryMock.Object
            );

            // Set up fixture for object creation
            fixture = new Fixture();
        }

        [TestMethod]
        [DataRow(1, false)]
        [DataRow(2, false)]
        public async Task IsBookingInHoliday_NullFromHolidayDb_ExpectedResultFalse(int noOfDays, bool expectedResult)
        {
            // Arrange
            var startDate = DateTime.Today.ToUniversalTime();
            var endDate = DateTime.Today.AddDays(noOfDays).ToUniversalTime();

            holidayRepositoryMock
            .Setup(mock => mock.GetByPredicate(It.IsAny<DateTime>(), It.IsAny<DateTime>())).ReturnsAsync((IEnumerable<Holiday>)null);

            // Act
            var result = await repository.IsBookingInHoliday(startDate, endDate);

            // Assert
            result.Should().Be(expectedResult);
        }
        
    }
}