using System.Net.WebSockets;
using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using MimeKit;
using MailKit;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using unit_tests.Utilities;
using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Models.Common;
using Nager.Date;

namespace unit_tests
{
    [TestClass]
    public class HolidayRepositoryTests
    {
        private Mock<ITableStorageClient<Holiday>> clientMock;
        private Mock<IHolidayMapper> holidayMapperMock;
        private HolidayRepository repository;
        private Fixture fixture;


        [TestInitialize]
        public void TestInit()
        {
            // Set up mock dependencies
            clientMock = new Mock<ITableStorageClient<Holiday>>();
            holidayMapperMock = new Mock<IHolidayMapper>();
            repository = new HolidayRepository
            (
                // Add .Object to make them correct type for class/constructor
                clientMock.Object,
                holidayMapperMock.Object
            );

            // Set up fixture for object creation
            fixture = new Fixture();
        }

        [TestMethod]
        public void AddWeekAsHoliday_CheckNoOfDays_ReturnSeven()
        {
            // Arrange
            var input = 1;
            var expectedResult = 7;

            // Act
            var result = repository.AddWeekAsHoliday(input, DateTime.Today.Year);

            // Assert
            result.Count().Equals(expectedResult);
        }
    }
}