import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';
import { HomeComponent } from './views/home/home.component';
import { MsalGuard } from '@azure/msal-angular';
import { RentalObjectComponent } from './views/rental-object/rental-object.component';
import { RegisterNewRentalObjectComponent } from './views/register-new-rental-object/register-new-rental-object.component';
import { BookingOverviewComponent } from './views/booking-overview/booking-overview.component';
import { NotFoundComponent } from './views/not-found/not-found.component';
import { AdminChecklistviewComponent } from './views/admin-checklistview/admin-checklistview.component';
import { AdminBookingOverviewComponent } from './views/admin-booking-overview/admin-booking-overview.component';
import { AdminUsermanagementComponent } from './views/admin-usermanagement/admin-usermanagement.component';

const routes: Routes = [
  {
    path: 'locations/new',
    component: RegisterNewRentalObjectComponent,
    canActivate: [MsalGuard],
  },
  {
    path: 'locations/new/:id',
    component: RegisterNewRentalObjectComponent,
    canActivate: [MsalGuard],
  },
  {
    path: 'locations/:id',
    component: RentalObjectComponent,
    canActivate: [MsalGuard]
  },
  {
    path: 'bookings/overview',
    component: AdminBookingOverviewComponent,
    canActivate: [MsalGuard]
  },
  {
    path: 'bookings',
    component: BookingOverviewComponent,
    canActivate: [MsalGuard],
  },
  {
    path: 'checklists',
    component: AdminChecklistviewComponent,
    canActivate: [MsalGuard]
  },
  {
    path: 'users',
    component: AdminUsermanagementComponent,
    canActivate: [MsalGuard],
  },
  // Home paths
  {
    path: '',
    component: HomeComponent,
    canActivate: [MsalGuard],
  },
  {
    // Needed for hash routing
    path: 'code',
    component: HomeComponent,
    canActivate: [MsalGuard],
  },
  // Not-found 404 paths
  {
    path: '404',
    component: NotFoundComponent,
  },
  {
    path: '**',
    redirectTo: '/404',
  }
];

const isIframe = window !== window.parent && !window.opener;

@NgModule({
  imports: [
    RouterModule.forRoot(routes, {
      // Don't perform initial navigation in iframes
      initialNavigation: !isIframe ? 'enabled' : 'disabled',
    }),
  ],
  exports: [RouterModule],
})
export class AppRoutingModule {}
