/* eslint-disable prefer-arrow/prefer-arrow-functions */
import { TestBed, waitForAsync } from '@angular/core/testing';
import { AppComponent } from './app.component';
import {
  MSAL_INSTANCE,
  MsalService,
  MsalModule,
  MsalInterceptor,
  MSAL_GUARD_CONFIG,
  MSAL_INTERCEPTOR_CONFIG,
  MsalGuard,
  MsalBroadcastService,
} from '@azure/msal-angular';
import {
  IPublicClientApplication,
  PublicClientApplication,
} from '@azure/msal-browser';
import { HomeComponent } from './views/home/home.component';
import { HeaderComponent } from './components/header/header.component';
import { RentalObjectComponent } from './views/rental-object/rental-object.component';
import {
  BsDatepickerModule,
  BsLocaleService,
  DatePickerComponent,
} from 'ngx-bootstrap/datepicker';
import { MapComponent } from './components/map/map.component';
import { RegisterNewRentalObjectComponent } from './views/register-new-rental-object/register-new-rental-object.component';
import { AppRoutingModule } from './app-routing.module';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { CarouselModule } from 'ngx-bootstrap/carousel';
import { CollapseModule } from 'ngx-bootstrap/collapse';
import { NgSelectModule } from '@ng-select/ng-select';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import {
  msalGuardConfigFactory,
  msalInterceptorConfigFactory,
} from './app.module';
import { MenuItems } from './models/menuItems.model';
import { HttpService } from './services/http.service';
import { BookingService } from './services/booking.service';
import { RentalObjectService } from './services/rentalObject.service';
import { UserService } from './services/user.service';
import { LOCALE_ID } from '@angular/core';
import { BsModalService } from 'ngx-bootstrap/modal';

function msalInstanceFactory(): IPublicClientApplication {
  return new PublicClientApplication({
    auth: {
      clientId: '6226576d-37e9-49eb-b201-ec1eeb0029b6',
      redirectUri: 'http://localhost:4200',
    },
  });
}

describe('AppComponent', () => {
  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        declarations: [
          AppComponent,
          HomeComponent,
          HeaderComponent,
          RentalObjectComponent,
          DatePickerComponent,
          MapComponent,
          RegisterNewRentalObjectComponent,
        ],
        imports: [
          AppRoutingModule,
          BrowserModule,
          BrowserAnimationsModule,
          BsDatepickerModule.forRoot(),
          CarouselModule.forRoot(),
          CollapseModule.forRoot(),
          NgSelectModule,
          FormsModule,
          HttpClientModule,
          MsalModule,
          ReactiveFormsModule,
        ],
        providers: [
          {
            provide: HTTP_INTERCEPTORS,
            useClass: MsalInterceptor,
            multi: true,
          },
          {
            provide: MSAL_INSTANCE,
            useFactory: msalInstanceFactory,
          },
          {
            provide: MSAL_GUARD_CONFIG,
            useFactory: msalGuardConfigFactory,
          },
          {
            provide: MSAL_INTERCEPTOR_CONFIG,
            useFactory: msalInterceptorConfigFactory,
          },
          BsLocaleService,
          MsalService,
          MsalGuard,
          MsalBroadcastService,
          MenuItems,
          HttpService,
          BookingService,
          RentalObjectService,
          UserService,
          { provide: LOCALE_ID, useValue: 'nb-NO' },
          BsModalService,
        ],
      }).compileComponents();
    })
  );

  it('should create the app', () => {
    const fixture = TestBed.createComponent(AppComponent);
    const app = fixture.componentInstance;
    expect(app).toBeTruthy();
  });

  /*
  it(`should have as title 'Utleie - Communicate'`, () => {
    const fixture = TestBed.createComponent(AppComponent);
    const app = fixture.componentInstance;
    expect(app.title).toEqual('Utleie - Communicate');
  });

  it('should render title', () => {
    const fixture = TestBed.createComponent(AppComponent);
    fixture.detectChanges();
    const compiled = fixture.nativeElement;
    expect(compiled.querySelector('.title').textContent).toContain('Utleie - Communicate');
  });
  */
});
