/* eslint-disable prefer-arrow/prefer-arrow-functions */
// --- Imports ---
// Angular imports
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { NgModule, LOCALE_ID } from '@angular/core';
import { registerLocaleData } from '@angular/common';
import { HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import localeNorwegian from '@angular/common/locales/nb';
registerLocaleData(localeNorwegian);

// Angular material imports
import { MatSidenavModule } from '@angular/material/sidenav';
import { MatListModule } from '@angular/material/list';

// ngx-bootstrap modules
import { AlertModule } from 'ngx-bootstrap/alert';
import { CarouselModule } from 'ngx-bootstrap/carousel';
import { CollapseModule } from 'ngx-bootstrap/collapse';
import { BsDatepickerModule } from 'ngx-bootstrap/datepicker';
import { BsModalService } from 'ngx-bootstrap/modal';
import { TooltipModule } from 'ngx-bootstrap/tooltip';
import { TimepickerModule } from 'ngx-bootstrap/timepicker';

// MSAL imports
import {
  IPublicClientApplication,
  PublicClientApplication,
  InteractionType,
  BrowserCacheLocation,
} from '@azure/msal-browser';
import {
  MsalGuard,
  MsalInterceptor,
  MsalBroadcastService,
  MsalInterceptorConfiguration,
  MsalModule,
  MsalService,
  MSAL_GUARD_CONFIG,
  MSAL_INSTANCE,
  MSAL_INTERCEPTOR_CONFIG,
  MsalGuardConfiguration,
} from '@azure/msal-angular';

// Other
import { BsLocaleService } from 'ngx-bootstrap/datepicker';
import { NgSelectModule } from '@ng-select/ng-select';
import { FullCalendarModule } from '@fullcalendar/angular';
import dayGridPlugin from '@fullcalendar/daygrid';

// Internal components
import { environment } from 'src/environments/environment';
import { AdminBookingOverviewComponent } from './views/admin-booking-overview/admin-booking-overview.component';
import { AdminChecklistviewComponent } from './views/admin-checklistview/admin-checklistview.component';
import { AdminChecklistviewModalComponent } from './components/admin-checklistview-modal/admin-checklistview-modal.component';
import { AppComponent } from './app.component';
import { AppRoutingModule } from './app-routing.module';
import { BookingOverviewComponent } from './views/booking-overview/booking-overview.component';
import { CancellationModalComponent } from './components/cancellation-modal/cancellation-modal.component';
import { ChecklistModalComponent } from './components/checklist-modal/checklist-modal.component';
import { DatePickerComponent } from './components/date-picker/date-picker.component';
import { FeedbackModalComponent } from './components/feedback-modal/feedback-modal.component';
import { HeaderComponent } from './components/header/header.component';
import { HomeComponent } from './views/home/home.component';
import { MapComponent } from './components/map/map.component';
import { MenuItems } from './models/menuItems.model';
import { NotFoundComponent } from './views/not-found/not-found.component';
import { RegisterNewRentalObjectComponent } from './views/register-new-rental-object/register-new-rental-object.component';
import { RentalObjectComponent } from './views/rental-object/rental-object.component';
import { AdminUsermanagementComponent } from './views/admin-usermanagement/admin-usermanagement.component';

// Internal services
import { BookingService } from './services/booking.service';
import { HttpService } from './services/http.service';
import { RentalObjectService } from './services/rentalObject.service';
import { UserService } from './services/user.service';
import { FeedbackService } from './services/feedback.service';
import { OverviewcalendarComponent } from './components/overviewcalendar/overviewcalendar.component';

// --- MSAL functionality ---
const isIE =
  window.navigator.userAgent.indexOf('MSIE ') > -1 ||
  window.navigator.userAgent.indexOf('Trident/') > -1;

export function msalInstanceFactory(): IPublicClientApplication {
  return new PublicClientApplication({
    auth: {
      clientId: environment.clientId,
      authority: 'https://login.microsoftonline.com/' + environment.tenantId,
      redirectUri: environment.baseUrl,
    },
    cache: {
      cacheLocation: BrowserCacheLocation.LocalStorage,
      storeAuthStateInCookie: isIE, // set to true for IE 11
    },
  });
}
export function msalInterceptorConfigFactory(): MsalInterceptorConfiguration {
  const protectedResourceMap = new Map<string, Array<string>>();
  protectedResourceMap.set('https://graph.microsoft.com/v1.0/me', [
    'user.read',
  ]);

  return {
    interactionType: InteractionType.Redirect,
    protectedResourceMap,
  };
}
export function msalGuardConfigFactory(): MsalGuardConfiguration {
  return { interactionType: InteractionType.Redirect };
}


// Register FullCalendar plugins
FullCalendarModule.registerPlugins([
  dayGridPlugin
]);

// --- Angular imports, declarations etc ---
@NgModule({
  declarations: [
    AdminChecklistviewComponent,
    AdminChecklistviewModalComponent,
    AppComponent,
    BookingOverviewComponent,
    CancellationModalComponent,
    ChecklistModalComponent,
    DatePickerComponent,
    FeedbackModalComponent,
    HeaderComponent,
    HomeComponent,
    MapComponent,
    NotFoundComponent,
    RegisterNewRentalObjectComponent,
    RentalObjectComponent,
    AdminBookingOverviewComponent,
    AdminUsermanagementComponent,
    OverviewcalendarComponent,
  ],
  imports: [
    AlertModule.forRoot(),
    AppRoutingModule,
    BrowserAnimationsModule,
    BrowserModule,
    BsDatepickerModule.forRoot(),
    CarouselModule.forRoot(),
    CollapseModule.forRoot(),
    FormsModule,
    FullCalendarModule,
    HttpClientModule,
    MatListModule,
    MatSidenavModule,
    MsalModule,
    NgSelectModule,
    ReactiveFormsModule,
    TooltipModule.forRoot(),
    TimepickerModule.forRoot(),
  ],
  providers: [
    {
      provide: HTTP_INTERCEPTORS,
      useClass: MsalInterceptor,
      multi: true,
    },
    {
      provide: MSAL_INSTANCE,
      useFactory: msalInstanceFactory,
    },
    {
      provide: MSAL_GUARD_CONFIG,
      useFactory: msalGuardConfigFactory,
    },
    {
      provide: MSAL_INTERCEPTOR_CONFIG,
      useFactory: msalInterceptorConfigFactory,
    },
    BookingService,
    BsLocaleService,
    { provide: LOCALE_ID, useValue: 'nb-NO' },
    BsModalService,
    HttpService,
    MenuItems,
    FeedbackService,
    MsalBroadcastService,
    MsalGuard,
    MsalService,
    RentalObjectService,
    UserService,
  ],
  bootstrap: [AppComponent],
})
export class AppModule {}
