import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BrowserDynamicTestingModule } from '@angular/platform-browser-dynamic/testing';
import { BsModalRef, BsModalService, ModalModule } from 'ngx-bootstrap/modal';
import { Booking } from 'src/app/models/booking.model';
import { BookingService } from 'src/app/services/booking.service';

import { BookingOverviewComponent } from 'src/app/views/booking-overview/booking-overview.component';
import { CancellationModalComponent } from './cancellation-modal.component';

describe('CancellationModalComponent', () => {
  let component: CancellationModalComponent;
  let fixture: ComponentFixture<CancellationModalComponent>;
  let bookingServiceMock: HttpTestingController;

  const bookingServiceSpy = jasmine.createSpyObj('bookingService', [
    'getAll',
    'getByUser',
    'getConfirmedByRentalObject',
    'createNew',
    'delete',
  ]);

  const bookingTestPost: Booking = {
    id: '1',
    userId: '2',
    englishTitle: 'Borrenose',
    title: 'Børrenøse',
    startDate: new Date('2021-03-18'),
    endDate: new Date('2021-04-03'),
    isConfirmed: true,
  };

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [ModalModule.forRoot(), HttpClientTestingModule],
      declarations: [CancellationModalComponent],
      providers: [
        BsModalService,
        BsModalRef,
        { provide: BookingService, useValue: bookingServiceSpy },
      ],
    })
      .overrideModule(BrowserDynamicTestingModule, {
        set: {
          entryComponents: [BookingOverviewComponent],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(CancellationModalComponent);
    component = fixture.componentInstance;
    component.booking = bookingTestPost;
    fixture.detectChanges();

    bookingServiceMock = TestBed.inject(HttpTestingController);
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });
});
