import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder } from '@angular/forms';
import { BrowserDynamicTestingModule } from '@angular/platform-browser-dynamic/testing';
import { BsModalRef, BsModalService, ModalModule } from 'ngx-bootstrap/modal';
import { of, Subject } from 'rxjs';
import { Booking } from 'src/app/models/booking.model';
import { ChecklistService } from 'src/app/services/checklist.service';
import { RentalObjectService } from 'src/app/services/rentalObject.service';
import { BookingOverviewComponent } from 'src/app/views/booking-overview/booking-overview.component';

import { ChecklistModalComponent } from './checklist-modal.component';

describe('ChecklistComponent', () => {
  let component: ChecklistModalComponent;
  let fixture: ComponentFixture<ChecklistModalComponent>;
  let rentalObjectServiceSpy: jasmine.SpyObj<RentalObjectService>;
  let checklistServiceSpy: jasmine.SpyObj<ChecklistService>;

  /*
  const rentalObjectServiceSpy = jasmine.createSpyObj("rentalObjectService", [
    "getAll",
    "getAllIds",
    "getByEnglishTitle",
    "getChecklist",
    "post",
    "delete"
  ]);
  */

  const rentalSpy = jasmine.createSpyObj<RentalObjectService>(
    'RentalObjectService',
    ['getChecklist']
  );
  const checklistSpy = jasmine.createSpyObj<ChecklistService>(
    'ChecklistService',
    ['send']
  );

  const bookingTestPost: Booking = {
    id: '1',
    userId: '2',
    englishTitle: 'Borrenose',
    title: 'Børrenøse',
    startDate: new Date('2021-03-18'),
    endDate: new Date('2021-04-03'),
    isConfirmed: true,
  };

  const checklist: string[] = ['Some item', 'Another item', 'More items'];

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [ModalModule.forRoot(), HttpClientTestingModule],
      declarations: [ChecklistModalComponent],
      providers: [
        BsModalService,
        BsModalRef,
        FormBuilder,
        { provide: RentalObjectService, useValue: rentalSpy },
        { provide: ChecklistService, useValue: checklistSpy },
      ],
    })
      .overrideModule(BrowserDynamicTestingModule, {
        set: {
          entryComponents: [BookingOverviewComponent],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(ChecklistModalComponent);
    component = fixture.componentInstance;
    component.booking = bookingTestPost;
    fixture.detectChanges();

    rentalObjectServiceSpy = TestBed.inject(
      RentalObjectService
    ) as jasmine.SpyObj<RentalObjectService>;
    checklistServiceSpy = TestBed.inject(
      ChecklistService
    ) as jasmine.SpyObj<ChecklistService>;

    rentalObjectServiceSpy.getChecklist.and.returnValue(
      new Subject<string[]>()
    );
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });
});
