import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import {
  FormArray,
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { BsModalRef } from 'ngx-bootstrap/modal';
import { Booking } from 'src/app/models/booking.model';
import { Checklist, ChecklistItem, CompletedChecklist } from 'src/app/models/checklist.model';
import { ChecklistService } from 'src/app/services/checklist.service';
import { UserService } from 'src/app/services/user.service';

@Component({
  selector: 'component-checklist-modal',
  templateUrl: './checklist-modal.component.html',
  styleUrls: ['./checklist-modal.component.scss'],
})
export class ChecklistModalComponent implements OnInit {
  @Output() checklistComplete = new EventEmitter<any>();
  booking: Booking;
  checklist: ChecklistItem[] = [] as any;
  fullChecklist: CompletedChecklist;
  control: FormControl;
  checklistForm: FormGroup;
  checklistLoaded = false;

  constructor(
    private modalRef: BsModalRef,
    private checklistService: ChecklistService,
    private fb: FormBuilder,
    private userService: UserService
  ) {}

  async ngOnInit() {
    this.checklistForm = this.fb.group({
      items: new FormArray([]),
      comment: new FormControl('')
    });

    await this.getCheckItems();
    this.addCheckboxes();
    this.comment.setValue(this.fullChecklist.comment);
  }

  async submit() {
    // Get checklist values and map to correct format
    const checkedBoxes = this.checklistForm.value.items.map(
      (checked: boolean, i: number) => ({
        description: this.checklist[i].description,
        completed: checked ? true : false,
      })
    );

    // Submit to backend
     this.checklistService
      .send(
        await this.userService.getUserId(),
        this.booking.id,
        this.booking.title,
        this.booking.startDate,
        this.booking.endDate,
        checkedBoxes,
        this.comment.value
      )
      .subscribe((completed) => {
        this.checklistComplete.emit(completed);
      });

    this.modalRef.hide();
  }

  closeModal() {
    this.modalRef.hide();
  }

  get comment() { return this.checklistForm.get('comment') as FormControl; }

  get checkListFormArray() {
    return this.checklistForm.controls.items as FormArray;
  }

  private addCheckboxes() {
    this.checklist.forEach((checkItem: ChecklistItem, i: number) => {
      this.checkListFormArray.push(new FormControl(this.checklist[i].completed));
    });
  }

  private async getCheckItems() {
    await this.checklistService
      .getChecklist(this.booking.englishTitle, this.booking.id)
      .toPromise()
      .then((result) => {
        this.fullChecklist = result;
        this.checklist = this.fullChecklist.filledChecklist;
        this.checklistLoaded = true;
      });
  }

}
