import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { AlertComponent } from 'ngx-bootstrap/alert';
import { BsModalRef } from 'ngx-bootstrap/modal';
import { Feedback } from 'src/app/models/feedback.model';
import { FeedbackService } from 'src/app/services/feedback.service';


@Component({
  selector: 'app-feedback-modal',
  templateUrl: './feedback-modal.component.html',
  styleUrls: ['./feedback-modal.component.scss']
})
export class FeedbackModalComponent implements OnInit {
  feedbackForm: FormGroup;
  alerts: any[] = [];

  constructor(
    private modalRef: BsModalRef,
    private feedbackService: FeedbackService
  ) { }

  ngOnInit(): void {
    this.feedbackForm = new FormGroup({
      pageInfo: new FormControl('', Validators.required),
      feedback: new FormControl('', Validators.required)
    });
  }

  get pageInfo() { return this.feedbackForm.get('pageInfo') as FormControl; }
  get feedback() { return this.feedbackForm.get('feedback') as FormControl; }

  submit(): void {
    this.feedbackService.sendFeedback(new Feedback(this.pageInfo.value, this.feedback.value))
    // Success alert on success
    .subscribe(res => {
      this.alerts.push({
        type: 'success',
        msg: 'Takk for tilbakemeldingen!',
        timeout: 2500
      });
    // Error alert on error
    }, error => {
      this.alerts.push({
        type: 'danger',
        msg: 'Kunne ikke sende tilbakemeldingen, vennligst prøv igjen senere.',
        timeout: 2500
      });
      console.log(error);
    });

    // Timeout equals alert timeout
    setTimeout(() => this.modalRef.hide(), 2500);
  }

  closeModal(): void {
    this.modalRef.hide();
  }

  onClosedAlert(dismissedAlert: AlertComponent): void {
    this.alerts = this.alerts.filter(alert => alert !== dismissedAlert);
  }

}
