import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { of } from 'rxjs';
import { TestBed } from '@angular/core/testing';
import { Booking } from '../models/booking.model';
import { BookingService } from './booking.service';
import { HttpService } from './http.service';

describe('BookingService', () => {
  let service: BookingService;
  let httpMock: HttpTestingController;
  let httpClient: HttpClientTestingModule;

  const httpServiceSpy = jasmine.createSpyObj('httpService', [
    'get',
    'delete',
    'post',
  ]);

  const bookingTestPost: Booking = {
    id: '1',
    userId: '2',
    englishTitle: 'Borrenose',
    title: 'Børrenøse',
    startDate: new Date('2021-03-18'),
    endDate: new Date('2021-04-03'),
    isConfirmed: true,
  };

  const bookingTestPostArray: Booking[] = [
    {
      id: '1',
      userId: '2',
      englishTitle: 'Borrenose',
      title: 'Børrenøse',
      startDate: new Date('2021-03-18'),
      endDate: new Date('2021-04-03'),
      isConfirmed: true,
    },
  ];

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        BookingService,
        { provide: HttpService, useValue: httpServiceSpy },
      ],
    }).compileComponents();

    service = TestBed.inject(BookingService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  it('should create an observable of type Booking via a GET call getAll()', () => {
    /*
        spyOn(service, 'getAll').and.returnValue(of(bookingTestPost));

        service.getAll().subscribe(posts => {
            expect(posts).toEqual(bookingTestPost);
            httpMock.verify();
        })
        */
  });

  it('should create an observable of type Booking via a GET call getByUser(userId: string)', () => {
    spyOn(service, 'getByUser').and.returnValue(of(bookingTestPostArray));

    service.getByUser('1').subscribe((posts) => {
      expect(posts).toEqual(bookingTestPostArray);
      httpMock.verify();
    });
  });

  it('should create an observable of type Booking via a GET call getConfirmedByRentalObject(locationTitle: string)', () => {
    spyOn(service, 'getConfirmedByRentalObject').and.returnValue(
      of(bookingTestPostArray)
    );

    service.getConfirmedByRentalObject('Borrenose').subscribe((posts) => {
      expect(posts).toEqual(bookingTestPostArray);
      httpMock.verify();
    });
  });

  it('should create an observable of type boolean via a POST call createNew' +
  '(userId: string, locationEnglishTitle: string, startDate: Date, endDate: Date)', () => {
    const testBool = true;

    /*
    spyOn(service, 'createNew').and.returnValue(of(testBool));
    service
      .createNew(
        '1',
        'Borrenose',
        'Børrenøse',
        new Date('2021-03-18'),
        new Date('2021-04-03')
      )
      .subscribe((posts) => {
        expect(posts).toEqual(testBool);
        httpMock.verify();
      });
      */
  });

  it('should create an observable of type Booking via a DELETE call delete(id: string)', () => {
    const testBool = true;

    spyOn(service, 'cancel').and.returnValue(of(testBool));
    service.cancel('1', 'Borrenose').subscribe((posts) => {
      expect(posts).toEqual(true);
      httpMock.verify();
    });
  });
});
