import { HttpClient } from '@angular/common/http';
import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { of } from 'rxjs';
import { TestBed } from '@angular/core/testing';
import { HttpService } from './http.service';
import { RentalObjectService } from './rentalObject.service';
import {
  PeriodPrice,
  RentalObject,
  RentalObjectId,
} from '../models/rentalObject.model';
import { AllocationPeriod } from '../models/allocationPeriod.model';

describe('RentalObjectService', () => {
  let service: RentalObjectService;
  let httpMock: HttpTestingController;
  let httpClient: HttpClientTestingModule;
  const httpServiceSpy = jasmine.createSpyObj('httpService', [
    'get',
    'delete',
    'post',
  ]);

  /*
  const rentalObjectTestPost: RentalObject = {
    englishTitle: 'Borrenose',
    title: 'Børrenøse',
    bedrooms: 3,
    beds: 4,
    sleeps: 6,
    description: 'Hyggelig hytte i Valdres',
    address: 'Børrenøsvegen 43, 2985 Tyinkrysset',
    petsAllowed: false,
    smokingAllowed: false,
    otherRules: [
      'Vær forsiktig med stearinslys.',
      'Vis hensyn overfor naboer mht. støy.',
      'Parkering på anvist sted.',
      'Nøkler oppbevares slik at de ikke kan identifiseres med hytta eller Communicate.',
    ],
    checkList: [
      'Testet røykvarslere.',
      'Rydded opp.',
      'Rengjort og fjernet matrester fra kjøleskap, stekeovn osv.',
      'Tømt søppel',
      '...',
    ],
    cardPicture: 'assets/pictures/borrenose/borrenose_4.jpeg',
    pictures: [
      'assets/pictures/borrenose/borrenose_1.jpeg',
      'assets/pictures/borrenose/borrenose_2.jpeg',
      'assets/pictures/borrenose/borrenose_3.jpeg',
    ],
    allocationPeriods: [
      new AllocationPeriod(
        new Date('2021-03-15'),
        new Date('2021-05-01'),
        new Date('2021-10-31')
      ),
      new AllocationPeriod(
        new Date('2021-09-15'),
        new Date('2021-11-01'),
        new Date('2021-04-30')
      ),
    ],
    applicationLimit: 2,
    periodPrices: [
      new PeriodPrice('Påsken', '1500kr per periode'),
      new PeriodPrice('En uke', '1500kr per uke'),
    ],
    disabledDays: [0, 1],
    rentalPeriods: [
      'Hyttebytte mellom to leiere foretas normalt på mandager. \nDen som avslutter leieforhold må være ute senest kl 12:00.' +
      '\nDen som starter leieforhold kan tidligst overta hytten kl 17:00.',
      'Påsken deles i 2 perioder: \nDel 1: Fredag før påske kl 17:00 – onsdag før påske kl 12:00' +
      '\nDel 2: Onsdag før påske kl 17:00 – mandag 2. påskedag kl 12:00.',
      'I Pinsen foretas hyttebytte mandag 2. pinsedag.' +
      '\nLeieperioden for pinsen er fra mandag før pinse kl 17:00 til mandag 2. pinsedag kl 12:00.',
      'Julen: Perioden kunngjøres på e-post.',
    ],
    latitude: 61.187065,
    longtitude: 8.213971,
    fireAlarm: ['en brannalarm', 'to brannalarm'],
    fireResponsible: 'Svein Veland',
    fireTestResponsible: 'Hver enkelt',
    fireDepartmentNumber: '110',
    fireAlarmNoFireInstructions:
      'Luft, vift med avis el.l for å fjerne røyken, eller trykk test/pauseknappen (midt på flaten) på den røykvarsleren som' +
      'utløste alarmen. Hvis du trenger lengre tid enn 10 minutter på å få ut røyken, trykker du test/pauseknappen en gang til' +
      'når den begynner å ule igjen. Hvis røyk ikke er årsaken til falsk alarm, støvsug varslerne utvendig langs åpningen inn mot' +
      'elektronikk og deteksjonskammer. Ikke åpne dekslet for å prøve rengjøre inne i varsleren. Ikke ta ut batteriene!',
    fireEquipment:
      '6 stk seriekoblede røykvarslere Modell Housegard Origo SA422WS',
    equipment:
      'Glass/kopper:\n' + 'Små kjøkkenglass 11 stk\n' + 'Vinglass 15 stk\n\n',
    directions: 'This section only shows up if it exists and is not empty.',
    otherPractialInformation:
      'Annen praktisk informasjon her. Feks. billeie, golf, skibakke osv.',
  };


  const rentalObjectTestList = [rentalObjectTestPost];

  const rentalObjectIdList: RentalObjectId[] = [
    new RentalObjectId('Borrenose', 'Børrenøse'),
    new RentalObjectId('josefinsesgate', 'Josefinesgate'),
    new RentalObjectId('valle-romano', 'Valle Romano'),
  ];

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        RentalObjectService,
        { provide: HttpService, useValue: httpServiceSpy },
      ],
    }).compileComponents();

    service = TestBed.inject(RentalObjectService);
    httpMock = TestBed.inject(HttpTestingController);
    httpClient = TestBed.inject(HttpClientTestingModule);
  });

  it('should create an observable of type RentalObject via a GET call getAll()', () => {
    spyOn(service, 'getAll').and.returnValue(of(rentalObjectTestList));

    service.getAll().subscribe((posts) => {
      expect(posts).toEqual(rentalObjectTestList);
      httpMock.verify();
    });
  });

  it('should create an observable of type RentalObjectIds via a GET call getAllIds()', () => {
    spyOn(service, 'getAllIds').and.returnValue(of(rentalObjectIdList));

    service.getAllIds().subscribe((posts: RentalObjectId[]) => {
      expect(posts).toEqual(rentalObjectIdList);
      httpMock.verify();
    });
  });

  it('should create an observable of type RentalObject via a POST call post(rentalObject: any)', () => {
    /*
        spyOn(service, 'create').and.returnValue(of(rentalObjectTestPost));
        service.create(rentalObjectTestPost).subscribe(posts => {
            expect(posts).toEqual(rentalObjectTestPost);
            httpMock.verify();
        })

  });

  it('should create an observable of type RentalObject via a DELETE call delete(id: string)', () => {
    /*
        spyOn(service, 'remove').and.returnValue(of(rentalObjectTestPost));

        service.remove("1").subscribe(posts => {
            expect(posts).toEqual(rentalObjectTestPost);
            httpMock.verify();
        })

  });
  */
});
