import { Component, OnInit, TemplateRef } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { AlertComponent } from 'ngx-bootstrap/alert';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { CancellationModalComponent } from 'src/app/components/cancellation-modal/cancellation-modal.component';
import { Booking } from 'src/app/models/booking.model';
import { BookingService } from 'src/app/services/booking.service';

@Component({
  selector: 'app-admin-booking-overview',
  templateUrl: './admin-booking-overview.component.html',
  styleUrls: ['./admin-booking-overview.component.scss']
})
export class AdminBookingOverviewComponent implements OnInit {
  alerts: any[] = [];
  bookings: Booking[];
  today: Date;
  isLoaded = false;
  commentForm: FormGroup;
  bookingToUpdate: Booking;

  modalRef: BsModalRef;

  constructor(
    private bookingService: BookingService,
    private modalService: BsModalService
  ) { }

  ngOnInit(): void {
    this.getBookings();
    this.today = new Date();
  }

  confimCancellation(booking: Booking): void {
    // Create cancellation modal, passing relevant booking
    const initialState = { booking };
    this.modalRef = this.modalService.show(CancellationModalComponent, {
      initialState,
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });

    // Listen to event from cancellation modal
    this.modalRef.content.cancellationCallComplete.subscribe(
      (completed: boolean) => {
        if (completed) {
          this.getBookings();
          this.alerts.push({
            type: 'success',
            msg: 'Bookingen har blitt kansellert.',
            timeout: 10000
          });
        } else {
          this.alerts.push({
            type: 'danger',
            msg: 'Kunne ikke oppdatere booking - prøv igjen.',
            timeout: 10000
        });
      }
    });
  }

  onClosedAlert(dismissedAlert: AlertComponent): void {
    this.alerts = this.alerts.filter(alert => alert !== dismissedAlert);
  }

  initializeForm() {
    this.commentForm = new FormGroup({
      comment: new FormControl('', [Validators.required]),
    });
  }

  submitComment(){;
    this.bookingToUpdate.adminComment = this.comment.value;
    console.log(this.bookingToUpdate.adminComment);
    this.bookingService.postComment(this.bookingToUpdate).subscribe(result => {
      console.log(result);
    });
  }

  get comment() { return this.commentForm.get('comment') as FormControl; }

  openCommentModal(modal: TemplateRef<any>, booking: Booking){
    this.initializeForm();
    this.bookingToUpdate = booking;
    if(this.bookingToUpdate.adminComment !== null){
      console.log(this.bookingToUpdate.adminComment);
      this.comment.setValue(booking.adminComment);
    }

    this.modalRef = this.modalService.show(modal, {
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });

  }


  private async getBookings() {
    this.bookingService
      .getAll()
      .subscribe(results => {
        this.bookings = results;
        this.bookings.forEach(booking => {
          // Set date format correctly
          booking.startDate = new Date(booking.startDate);
        });
        this.bookings = this.bookings.sort((a, b) => b.startDate.getTime() - a.startDate.getTime());
        this.isLoaded = true;
      }, error => {
        this.bookings = [];
        this.isLoaded = true;
      });
  }

}
