import { DecimalPipe } from '@angular/common';
import { Component, OnInit, TemplateRef } from '@angular/core';
import { FormControl, FormGroup, Validators, } from '@angular/forms';
import { NewUser } from 'src/app/models/user.model';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { UserService } from 'src/app/services/user.service';
import { defineLocale } from 'ngx-bootstrap/chronos';
import { BsLocaleService } from 'ngx-bootstrap/datepicker';
import { nbLocale } from 'ngx-bootstrap/locale';
defineLocale('nb', nbLocale);

@Component({
  selector: 'app-admin-usermanagement',
  templateUrl: './admin-usermanagement.component.html',
  styleUrls: ['./admin-usermanagement.component.scss'],
  providers: [DecimalPipe]
})
export class AdminUsermanagementComponent implements OnInit {
  locale = 'nb';
  today = new Date();
  modalState = '';
  tempUserIndex = 0;
  users: NewUser[] = new Array();
  newUser: NewUser;
  isLoaded = false;
  userForm: FormGroup;
  modalRef: BsModalRef;

  constructor(
    private localeService: BsLocaleService,
    private modalService: BsModalService,
    private userService: UserService
    ) {
    this.initializeForm();
    this.localeService.use(this.locale);
  }

  initializeForm(): void {
    this.userForm = new FormGroup({
      email: new FormControl('', [Validators.required, Validators.email, Validators.pattern('^[A-Za-z0-9._%+-]+@communicate.no$')]),
      points: new FormControl('', [Validators.maxLength(50),Validators.required]),
      hiredDate: new FormControl('', [Validators.maxLength(50),Validators.required]),
      isAdmin: new FormControl(false, [Validators.maxLength(50),Validators.required]),
    });
  }

  ngOnInit(): void {
    this.userService.getAllUsers().subscribe(result => {
      this.users = result;
      this.isLoaded = true;
    }, error => this.isLoaded = true
    );
  }

  get email() { return this.userForm.get('email') as FormGroup;}
  get points() { return this.userForm.get('points') as FormGroup;}
  get hiredDate() { return this.userForm.get('hiredDate') as FormGroup;}
  get isAdmin() { return this.userForm.get('isAdmin') as FormGroup;}

  addFieldValue() {
    this.modalRef.hide();
    this.newUser = this.userForm.value;
    this.userService.add(this.newUser).subscribe(
      () => {
        this.users.push(this.newUser);
      }
    );
  }

  updateFieldValue() {
    this.modalRef.hide();
    this.newUser = this.userForm.value;
    this.userService.update(this.newUser).subscribe(
      () => {
        this.users[this.tempUserIndex] = this.newUser;
      }
    );
  }

  confirmRemoval() {
    this.modalRef.hide();
    // Remove user
    this.userService.delete(this.users[this.tempUserIndex].email).subscribe(
      () => {
        this.users.splice(this.tempUserIndex, 1);
      }
    );
  }

  storeFieldValue(index: number) {
    this.tempUserIndex = index;
  }

  open(modal: TemplateRef<any>, modalState: string, user?: NewUser) {

    if (modalState === 'update') {
      this.modalState = 'update';
      if (user){
        this.userForm.setValue({
          email: user.email,
          points: user.points,
          hiredDate: new Date(user.hiredDate),
          isAdmin: user.isAdmin,
        });
      }
    } else if (modalState === 'delete') {
        this.modalState = 'delete';
    } else {
        this.modalState = 'new';
        // Make sure form is empty before opening
        this.initializeForm();
    }

    this.modalRef = this.modalService.show(modal , {
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });
  }


}
