import { HttpClientTestingModule } from '@angular/common/http/testing';
import { NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, inject, TestBed } from '@angular/core/testing';
import { BrowserDynamicTestingModule } from '@angular/platform-browser-dynamic/testing';
import { ModalModule } from 'ngx-bootstrap/modal';
import { Subject } from 'rxjs';

import { CancellationModalComponent } from 'src/app/components/cancellation-modal/cancellation-modal.component';
import { ChecklistModalComponent } from 'src/app/components/checklist-modal/checklist-modal.component';
import { GraphUser } from 'src/app/models/user.model';
import { BookingService } from 'src/app/services/booking.service';
import { UserService } from 'src/app/services/user.service';

import { BookingOverviewComponent } from './booking-overview.component';

describe('BookingOverviewComponent', () => {
  let component: BookingOverviewComponent;
  let fixture: ComponentFixture<BookingOverviewComponent>;
  let bookingService: jasmine.SpyObj<BookingService>;
  let userService: jasmine.SpyObj<UserService>;

  const userId = 'someUserId';

  beforeEach(() => {
    bookingService = jasmine.createSpyObj<BookingService>('bookingService', [
      'getAll',
      'getByUser',
      'getConfirmedByRentalObject',
      'createNew',
      'delete',
    ]);

    userService = jasmine.createSpyObj<UserService>('userService', [
      'getUserId',
      'getUserName',
      'getUserPoints',
      'postUser',
    ]);

    TestBed.configureTestingModule({
      imports: [ModalModule.forRoot(), HttpClientTestingModule],
      providers: [
        { provide: BookingService, useValue: bookingService },
        { provide: UserService, useValue: userService },
      ],
      declarations: [CancellationModalComponent, ChecklistModalComponent],
      schemas: [NO_ERRORS_SCHEMA],
    })
      .overrideModule(BrowserDynamicTestingModule, {
        set: {
          entryComponents: [BookingOverviewComponent],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(BookingOverviewComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', inject([UserService], () => {
    expect(component).toBeTruthy();
  }));
});
