import { ChangeDetectorRef, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormArray, FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { AlertComponent } from 'ngx-bootstrap/alert';
import { Subscription } from 'rxjs/internal/Subscription';
import { AllocationPeriod } from 'src/app/models/allocationPeriod.model';
import { PeriodPrice, RentalObject } from 'src/app/models/rentalObject.model';
import { RentalObjectService } from 'src/app/services/rentalObject.service';
import { defineLocale } from 'ngx-bootstrap/chronos';
import { BsLocaleService } from 'ngx-bootstrap/datepicker';
import { nbLocale } from 'ngx-bootstrap/locale';
defineLocale('nb', nbLocale);
@Component({
  selector: 'app-register-new-rental-object',
  templateUrl: './register-new-rental-object.component.html',
  styleUrls: ['./register-new-rental-object.component.scss'],
})
export class RegisterNewRentalObjectComponent implements OnInit {
  @ViewChild('cardInput') cardInput: ElementRef;
  locale = 'nb';
  rentalForm: FormGroup;
  rentalObject: RentalObject;
  alerts: any[] = [];
  step = 1;
  isEdit = false;
  cardPictureEdited = false;
  myFiles: string[] = [];
  today = new Date();
  address: string[];
  daysChooser: any[] = [
    { id: 0, name: 'Mandag' },
    { id: 1, name: 'Tirsdag' },
    { id: 2, name: 'Onsdag' },
    { id: 3, name: 'Torsdag' },
    { id: 4, name: 'Fredag' },
    { id: 5, name: 'Lørdag' },
    { id: 6, name: 'Søndag' },
  ];

  numArraySelect: number[] = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
  private routeSub: Subscription;
  constructor(
    private cd: ChangeDetectorRef,
    private route: ActivatedRoute,
    private rentalObjectService: RentalObjectService,
    private router: Router,
    private localeService: BsLocaleService
    ) {
      this.localeService.use(this.locale);
    }

  ngOnInit(): void {
    this.initializeForm();
    this.routeSub = this.route.params.subscribe(async (params) => {
      // Get rentalobject and check that exists
      if (params.id){
        const result = await this.rentalObjectService.getByEnglishTitle(params.id);
        if (result) {
          this.rentalObject = result;

          this.onEdit();
        }
      }
    });
  }

  initializeForm(): void {
    this.rentalForm = new FormGroup ({
      formOne: new FormGroup({
        title: new FormControl('', [Validators.maxLength(50), Validators.required]),
        englishTitle: new FormControl('', [Validators.maxLength(50), Validators.required, Validators.pattern('^[a-zA-Z0-9_-]{1,}$')]),
        street: new FormControl('', [Validators.maxLength(50), Validators.required]),
        zip: new FormControl('', [ Validators.maxLength(8), Validators.required]),
        city: new FormControl('', [Validators.maxLength(60), Validators.required]),
        latitude: new FormControl(null, [Validators.pattern
          ('^(\\+|-)?(?:90(?:(?:\\.0{1,6})?)|(?:[0-9]|[1-8][0-9])(?:(?:\\.[0-9]{1,6})?))$'),Validators.required]),
        longtitude: new FormControl(null, [Validators.pattern
          ('^(\\+|-)?(?:180(?:(?:\\.0{1,6})?)|(?:[0-9]|[1-9][0-9]|1[0-7][0-9])(?:(?:\\.[0-9]{1,6})?))$'), Validators.required]),
        sleeps: new FormControl(null, Validators.required),
        beds: new FormControl(null, Validators.required),
        bedrooms: new FormControl(null, Validators.required),
        directions: new FormControl('', [Validators.maxLength(5000), Validators.required]),
      }),
      formTwo: new FormGroup ({
        petsAllowed: new FormControl(false),
        smokingAllowed: new FormControl(false),
        otherRules: new FormArray([new FormControl(null, Validators.maxLength(200))]),
        checkList: new FormArray([new FormControl(null, Validators.maxLength(250))]),
        equipment: new FormControl('', [Validators.maxLength(5000), Validators.required]),
        otherPractialInformation: new FormControl('', Validators.maxLength(5000)),
        beforeYouTravel: new FormArray([ new FormGroup({
          title: new FormControl(null, Validators.maxLength(50)),
          description: new FormControl(null, Validators.maxLength(500)),
        }) ]),
      }),
      formThree: new FormGroup ({
        fireResponsible: new FormControl('', [Validators.maxLength(50), Validators.required]),
        fireTestResponsible: new FormControl('', [Validators.maxLength(50), Validators.required]),
        fireDepartmentNumber: new FormControl('', [Validators.pattern('[- +()0-9]+'), Validators.maxLength(14) , Validators.required]),
        fireAlarmNoFireInstructions: new FormControl('', [Validators.maxLength(5000), Validators.required]),
        fireAlarm: new FormArray([new FormControl(null, [Validators.maxLength(150), Validators.required])]),
        fireEquipment: new FormControl('', [Validators.maxLength(5000), Validators.required]),
      }),
      formFour: new FormGroup ({
        cardPicture: new FormControl(null, Validators.required),
        pictures: new FormArray([], Validators.required),
        description: new FormControl('', [Validators.maxLength(5000), Validators.required]),
      }),
      formFive: new FormGroup ({
        allocationPeriods: new FormArray([ new FormGroup({
          applicationDeadline: new FormControl(null),
          allocationStart: new FormControl(null),
          allocationEnd: new FormControl(null),
        }) ]),
        applicationLimit: new FormControl(null),
        periodPrices: new FormArray([ new FormGroup({
          period: new FormControl('', [Validators.maxLength(50), Validators.required]),
          price: new FormControl('', [Validators.maxLength(30), Validators.required]),
        }) ]),
        disabledDays: new FormControl(new Array<number>()),
        rentalPeriodsInnDay: new FormControl('', Validators.required),
        rentalPeriodsInnTime: new FormControl(null, Validators.required),
        rentalPeriodsOutDay: new FormControl('', Validators.required),
        rentalPeriodsOutTime: new FormControl(null, Validators.required),
        rentalPeriods: new FormArray([new FormControl('', Validators.maxLength(200))]),
      }),
    });
  }

  //Form groups
  get formOne() { return this.rentalForm.get('formOne') as FormGroup;}
  get formTwo() { return this.rentalForm.get('formTwo') as FormGroup;}
  get formThree() { return this.rentalForm.get('formThree') as FormGroup;}
  get formFour() { return this.rentalForm.get('formFour') as FormGroup;}
  get formFive() { return this.rentalForm.get('formFive') as FormGroup;}

  //FormOne
  get title() { return this.formOne.get('title') as FormControl; }
  get englishTitle() { return this.formOne.get('englishTitle') as FormControl; }
  get street() { return this.formOne.get('street') as FormControl; }
  get zip() { return this.formOne.get('zip') as FormControl; }
  get city() { return this.formOne.get('city') as FormControl; }
  get latitude() { return this.formOne.get('latitude') as FormControl; }
  get longtitude() { return this.formOne.get('longtitude') as FormControl; }
  get sleeps() { return this.formOne.get('sleeps') as FormControl; }
  get beds() { return this.formOne.get('beds') as FormControl; }
  get bedrooms() { return this.formOne.get('bedrooms') as FormControl; }
  get directions() { return this.formOne.get('directions') as FormControl; }

  //FormTwo
  get petsAllowed() { return this.formTwo.get('petsAllowed') as FormControl; }
  get smokingAllowed() { return this.formTwo.get('smokingAllowed') as FormControl; }
  get otherRules(): FormArray { return this.formTwo.get('otherRules') as FormArray; }
  get checkList(): FormArray { return this.formTwo.get('checkList') as FormArray; }
  get equipment() { return this.formTwo.get('equipment') as FormControl; }
  get otherPractialInformation() { return this.formTwo.get('otherPractialInformation') as FormControl; }
  get beforeYouTravel(): FormArray { return this.formTwo.get('beforeYouTravel') as FormArray; }

  //FormThree
  get fireResponsible() { return this.formThree.get('fireResponsible') as FormControl; }
  get fireTestResponsible() { return this.formThree.get('fireTestResponsible') as FormControl; }
  get fireDepartmentNumber() { return this.formThree.get('fireDepartmentNumber') as FormControl; }
  get fireAlarmNoFireInstructions() { return this.formThree.get('fireAlarmNoFireInstructions') as FormControl; }
  get fireEquipment() { return this.formThree.get('fireEquipment') as FormControl; }
  get fireAlarm(): FormArray { return this.formThree.get('fireAlarm') as FormArray; }

  //FormFour
  get cardPicture() { return this.formFour.get('cardPicture') as FormControl;}
  get pictures(): FormArray { return this.formFour.get('pictures') as FormArray; }
  get description() { return this.formFour.get('description') as FormControl; }

  //FormFive
  get allocationPeriods(): FormArray { return this.formFive.get('allocationPeriods') as FormArray; }
  get applicationLimit() { return this.formFive.get('applicationLimit') as FormControl; }
  get periodPrices(): FormArray { return this.formFive.get('periodPrices') as FormArray; }
  get disabledDays() { return this.formFive.get('disabledDays') as FormControl; }
  get rentalPeriodsInnDay() { return this.formFive.get('rentalPeriodsInnDay') as FormControl; }
  get rentalPeriodsInnTime() { return this.formFive.get('rentalPeriodsInnTime') as FormControl; }
  get rentalPeriodsOutDay() { return this.formFive.get('rentalPeriodsOutDay') as FormControl; }
  get rentalPeriodsOutTime() { return this.formFive.get('rentalPeriodsOutTime') as FormControl; }
  get rentalPeriods(): FormArray { return this.formFive.get('rentalPeriods') as FormArray; }

  // Checks each form validity on different steps
  checkNextButtonEnabled(): boolean {
    switch (this.step) {
      case 1:
        return this.formOne.valid;
      case 2:
        return this.formTwo.valid;
      case 3:
        return this.formThree.valid;
      case 4:
        return this.formFour.valid;
      case 5:
        return this.formFive.valid;
      default: return false;
    }
  }

  onNextStep(): void {
    this.step += 1;
  }

  onPreviousStep(): void {
    this.step -= 1;
  }

  // Adds items to form arrays
  addChecklistItem(): void {
    this.checkList.push(new FormControl(null));
  }

  addRule(): void {
    this.otherRules.push(new FormControl(null));
  }

  addRentalPeriods(): void {
    this.rentalPeriods.push(new FormControl(null));
}

  addfireAlarm(): void {
    this.fireAlarm.push(new FormControl(null, Validators.required));
  }

  addPictures(): void {
    this.pictures.push(new FormControl(null, Validators.required));
  }

  addPeriodPrices(): void {
    this.periodPrices.push(
      new FormGroup({
        period: new FormControl('', Validators.required),
        price: new FormControl('', Validators.required),
      })
    );
  }

  addAllocationPeriod(): void {
    this.allocationPeriods.push(
      new FormGroup({
        applicationDeadline: new FormControl(null, Validators.required),
        allocationStart: new FormControl(null, Validators.required),
        allocationEnd: new FormControl(null, Validators.required),
      })
    );
  }

  addBeforeYouTravel(): void {
    this.beforeYouTravel.push(
      new FormGroup({
        title: new FormControl(null, Validators.required),
        description: new FormControl(null, Validators.required),
      })
    );
  }

  // Removes items from form arrays
  removeChecklistItem(index: number): void {
    this.checkList.removeAt(index);
  }

  removeRule(index: number): void {
    this.otherRules.removeAt(index);
  }

  removeRentalPeriods(index: number): void {
    this.rentalPeriods.removeAt(index);
  }

  removefireAlarm(index: number): void {
    this.fireAlarm.removeAt(index);
  }

  removePictures(index: number): void {
    this.fireAlarm.removeAt(index);
  }

  removePeriodPrices(index: number): void {
    this.periodPrices.removeAt(index);
  }

  removeAllocationPeriods(index: number): void {
    this.allocationPeriods.removeAt(index);
  }

  removeBeforeYouTravel(index: number): void {
    this.beforeYouTravel.removeAt(index);
  }

  // Theese four func under is for file upload
  // Takes an event from file upload
  onFileSelected(event: any): void{
    for (const file of event.target.files) {
      this.pictures.push(new FormControl(file, Validators.required));
    }
  }

  onSingleFileSelected(event: any): void{
    this.cardPicture.setValue(event.target.files[0]);
    if(this.isEdit){
      this.cardPictureEdited = true;
    }
  }
  // Removes files from form
  removeSelectedFile(index: number): void {
    this.pictures.removeAt(index);
  }

  removeSelectedSingleFile(): void {
    this.cardPicture.setValue(null);
  }

  onClosedAlert(dismissedAlert: AlertComponent): void {
    this.alerts = this.alerts.filter(alert => alert !== dismissedAlert);
  }
  // Post new rentalObject
  async confirmPost(rentalObject: FormData) {
    this.rentalObjectService.create(rentalObject).subscribe(
      res => {console.log(res);
      if (res.ok){
        this.router.navigate(['/']);  // define your component where you want to go
      }
      else {
        this.alerts.push({
          type: 'error',
          msg: 'Kunne ikke oprette utleie objektet, vennligst prøv igjen senere.',
          timeout: 2500
        });
      }
    });
  }
  // Update new rentalObject
  async confirmUpdate(rentalObject: FormData) {
    this.rentalObjectService.update(rentalObject).subscribe(result => {
      console.log(result);
      if(result){
        this.router.navigate(['/']);
      } else {
        this.alerts.push({
          type: 'error',
          msg: 'Kunne ikke oppdatere utleie objektet, vennligst prøv igjen senere.',
          timeout: 2500
        });
      }
    });
  }

  // Mapping the values to the object to be posted
  async onSubmit() {
    const rentalObject = new FormData();

    this.address = [
      this.street.value,
      this.zip.value + ' '
      + this.city.value
    ];

    // Looping over all arrays to make them behave properly as enumerated objects in FormData
    rentalObject.append('address', this.address.join(', '));
    this.allocationPeriods.value.forEach((period: AllocationPeriod, i: number) => {
      rentalObject.append('allocationPeriods' + '[' + i + ']' + '.' + 'applicationDeadline', period.applicationDeadline.toISOString());
      rentalObject.append('allocationPeriods' + '[' + i + ']' + '.' + 'allocationStart', period.allocationStart.toISOString());
      rentalObject.append('allocationPeriods' + '[' + i + ']' + '.' + 'allocationEnd', period.allocationEnd.toISOString());
    });
    rentalObject.append('applicationLimit', this.applicationLimit.value);
    rentalObject.append('bedrooms', this.bedrooms.value);
    rentalObject.append('beds', this.beds.value);
    this.beforeYouTravel.value.forEach((info: any, i: number) => {
      rentalObject.append('beforeYouTravel' + '[' + i + ']' + '.' + 'title', info.title);
      rentalObject.append('beforeYouTravel' + '[' + i + ']' + '.' + 'description', info.description);
    });

    if (typeof this.cardPicture == 'string') {
      rentalObject.append('existingPictures', this.cardPicture);
    } else {
      rentalObject.append('cardPicture', this.cardPicture.value);
    }

    if (this.cardPictureEdited){
      rentalObject.append('cardPicture', this.cardPicture.value);
    } else {
      rentalObject.append('existingCardPicture', this.cardPicture.value);
    }

    this.checkList.value.forEach((checklistItem: string, i: number) => {
      rentalObject.append('checkList' + '[' + i + ']', checklistItem);
    });

    rentalObject.append('description', this.description.value);
    rentalObject.append('directions', this.directions.value);
    this.disabledDays.value.forEach((day: number, i: number) => {
      rentalObject.append('disabledDays' + '[' + i + ']', day.toString());
    });
    rentalObject.append('englishTitle', this.englishTitle.value.toLowerCase());
    rentalObject.append('equipment', this.equipment.value);
    this.fireAlarm.value.forEach((firealarm: string, i: number) => {
      rentalObject.append('fireAlarm' + '[' + i + ']', firealarm);
    });
    rentalObject.append('fireAlarmNoFireInstructions', this.fireAlarmNoFireInstructions.value);
    rentalObject.append('fireDepartmentNumber', this.fireDepartmentNumber.value);
    rentalObject.append('fireEquipment', this.fireEquipment.value);
    rentalObject.append('fireResponsible', this.fireResponsible.value);
    rentalObject.append('fireTestResponsible', this.fireTestResponsible.value);
    rentalObject.append('latitude', this.latitude.value);
    rentalObject.append('longtitude', this.longtitude.value);
    rentalObject.append('otherPractialInformation', this.otherPractialInformation.value);
    this.otherRules.value.forEach((rule: string, i: number) => {
      rentalObject.append('otherRules'  + '[' + i + ']', rule);
    });
    this.periodPrices.value.forEach((period: PeriodPrice, i: number) => {
      rentalObject.append('periodPrices' + '[' + i + ']' + '.' + 'period', period.period);
      rentalObject.append('periodPrices' + '[' + i + ']' + '.' + 'price', period.price);
    });
    rentalObject.append('petsAllowed', this.petsAllowed.value);
    this.pictures.value.forEach((image: any) => {
      if (typeof image == 'string') {
        rentalObject.append('existingPictures', image);
      } else {
        rentalObject.append('pictures', image);
      }
    });

    const inFormated = new Date(this.rentalPeriodsInnTime.value).getHours() + ':' +
      new Date(this.rentalPeriodsInnTime.value).getMinutes();
    const outFormated = new Date(this.rentalPeriodsOutTime.value).getHours() + ':' +
      new Date(this.rentalPeriodsOutTime.value).getMinutes();

    rentalObject.append('rentalPeriodsInnTime', inFormated);
    rentalObject.append('rentalPeriodsInnDay', this.rentalPeriodsInnDay.value);
    rentalObject.append('rentalPeriodsOutTime', outFormated);
    rentalObject.append('rentalPeriodsOutDay', this.rentalPeriodsOutDay.value);
    this.rentalPeriods.value.forEach((period: string, i: number) => {
      rentalObject.append('rentalPeriods' + '[' + i + ']', period);
    });
    rentalObject.append('sleeps', this.sleeps.value);
    rentalObject.append('smokingAllowed', this.smokingAllowed.value);
    rentalObject.append('title', this.title.value);

    if (this.isEdit === true){
      console.log(this.rentalForm);
      this.confirmUpdate(rentalObject);
    }
    else {
      this.confirmPost(rentalObject);
    }
  }

  //Called when the url contains a rentalObject id(englishTitle), sets all form values
  onEdit(){
    this.isEdit = true;

    for (let i = 0; i < this.rentalObject.otherRules.length - 1; i++) {
      this.addRule();
    }

    for (let i = 0; i < this.rentalObject.checkList.length - 1; i++) {
      this.addChecklistItem();
    }

    for (const file of this.rentalObject.pictures) {
      this.pictures.push(new FormControl(file, Validators.required));
    }

    for (let i = 0; i < this.rentalObject.periodPrices.length - 1; i++) {
      this.addPeriodPrices();
    }

    for (let i = 0; i < this.rentalObject.allocationPeriods.length; i++) {
      // Give periods correct format
      this.rentalObject.allocationPeriods[i].applicationDeadline = new Date(this.rentalObject.allocationPeriods[i].applicationDeadline);
      this.rentalObject.allocationPeriods[i].allocationStart = new Date(this.rentalObject.allocationPeriods[i].allocationStart);
      this.rentalObject.allocationPeriods[i].allocationEnd = new Date(this.rentalObject.allocationPeriods[i].allocationEnd);

      // There is already one in input array, so don't add extra on last round
      if (i < this.rentalObject.allocationPeriods.length - 1) {
        this.addAllocationPeriod();
      }
    }

    for (let i = 0; i < this.rentalObject.fireAlarm.length - 1; i++) {
      this.addfireAlarm();
    }

    for (let i = 0; i < this.rentalObject.beforeYouTravel.length - 1; i++) {
      this.addBeforeYouTravel();
    }

    for (let i = 0; i < this.rentalObject.rentalPeriods.length - 1; i++) {
      this.addRentalPeriods();
    }

    // Must ignore linter errors because they don't know what hours/minute properties are, even though they are present
    const innTimeValues = new Date();
    // @ts-ignore
    innTimeValues.setHours(this.rentalObject.rentalPeriodsInnTime.hours, this.rentalObject.rentalPeriodsInnTime.minutes, 0);
    const outTimeValues = new Date();
    // @ts-ignore
    outTimeValues.setHours(this.rentalObject.rentalPeriodsOutTime.hours, this.rentalObject.rentalPeriodsOutTime.minutes, 0);

    // *********************************************************************
    this.title.setValue(this.rentalObject.title);
    this.englishTitle.setValue(this.rentalObject.englishTitle);
    this.address = this.rentalObject.address.split(', ', 2);
    this.street.setValue(this.address[0]);
    this.zip.setValue(this.address[1].split(' ', 2)[0]);
    this.city.setValue(this.address[1].split(' ', 2)[1]);
    this.latitude.setValue(this.rentalObject.latitude);
    this.longtitude.setValue(this.rentalObject.longtitude);
    this.sleeps.setValue(this.rentalObject.sleeps);
    this.beds.setValue(this.rentalObject.beds);
    this.bedrooms.setValue(this.rentalObject.bedrooms);
    this.directions.setValue(this.rentalObject.directions);

    this.petsAllowed.setValue(this.rentalObject.petsAllowed);
    this.smokingAllowed.setValue(this.rentalObject.smokingAllowed);
    if (!this.isNullOrEmpty(this.rentalObject.otherRules)) { this.otherRules.setValue(this.rentalObject.otherRules); };
    if (!this.isNullOrEmpty(this.rentalObject.checkList)) { this.checkList.setValue(this.rentalObject.checkList); };
    this.equipment.setValue(this.rentalObject.equipment);
    if (this.rentalObject.otherPractialInformation) { this.otherPractialInformation.setValue(this.rentalObject.otherPractialInformation); };
    if (!this.isNullOrEmpty(this.rentalObject.beforeYouTravel)) { this.beforeYouTravel.setValue(this.rentalObject.beforeYouTravel); };

    if(!this.isNullOrEmpty(this.rentalObject.fireAlarm)) { this.fireAlarm.setValue(this.rentalObject.fireAlarm); };
    this.fireResponsible.setValue(this.rentalObject.fireResponsible);
    this.fireTestResponsible.setValue(this.rentalObject.fireTestResponsible);
    this.fireDepartmentNumber.setValue(this.rentalObject.fireDepartmentNumber);
    this.fireAlarmNoFireInstructions.setValue(this.rentalObject.fireAlarmNoFireInstructions);
    this.fireEquipment.setValue(this.rentalObject.fireEquipment);

    this.cardPicture.setValue(this.rentalObject.cardPicture);
    this.description.setValue(this.rentalObject.description);

    if (!this.isNullOrEmpty(this.rentalObject.allocationPeriods)) { this.allocationPeriods.setValue(this.rentalObject.allocationPeriods); };
    if (this.rentalObject.applicationLimit) {
      this.applicationLimit.setValue(this.rentalObject.applicationLimit);
    };
    if(!this.isNullOrEmpty(this.rentalObject.periodPrices)) { this.periodPrices.setValue(this.rentalObject.periodPrices); };
    if (this.rentalObject.disabledDays) { this.disabledDays.setValue(this.rentalObject.disabledDays); };
    this.rentalPeriodsInnDay.setValue(this.rentalObject.rentalPeriodsInnDay);
    this.rentalPeriodsInnTime.setValue(innTimeValues);
    this.rentalPeriodsOutDay.setValue(this.rentalObject.rentalPeriodsOutDay);
    this.rentalPeriodsOutTime.setValue(outTimeValues);
    if (!this.isNullOrEmpty(this.rentalObject.rentalPeriods)) { this.rentalPeriods.setValue(this.rentalObject.rentalPeriods); };
  }

  openImgNewTab(image: string) {
    window.open(image);
  }

  private isNullOrEmpty<T>(array: T[]): boolean {
      return !array || array.length === 0;
  }

}
