import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ActivatedRoute } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { BsModalService, ModalModule } from 'ngx-bootstrap/modal';
import { AllocationPeriod } from 'src/app/models/allocationPeriod.model';
import { Booking } from 'src/app/models/booking.model';
import { PeriodPrice, RentalObject } from 'src/app/models/rentalObject.model';
import { AllocationService } from 'src/app/services/allocation.service';
import { BookingService } from 'src/app/services/booking.service';
import { RentalObjectService } from 'src/app/services/rentalObject.service';
import { UserService } from 'src/app/services/user.service';
import { RentalObjectComponent } from './rental-object.component';

describe('RentalObjectComponent', () => {
  let component: RentalObjectComponent;
  let fixture: ComponentFixture<RentalObjectComponent>;
  let allocationServiceMock: HttpTestingController;
  let bookingServiceMock: HttpTestingController;
  let rentalObjectServiceMock: HttpTestingController;
  let userServiceMock: HttpTestingController;

  const allocationServiceSpy = jasmine.createSpyObj('allocationService', [
    'getNext',
  ]);

  const bookingServiceSpy = jasmine.createSpyObj('bookingService', [
    'getAll',
    'getByUser',
    'getConfirmedByRentalObject',
    'createNew',
    'delete',
  ]);

  const rentalObjectServiceSpy = jasmine.createSpyObj('rentalObjectService', [
    'getAll',
    'getAllIds',
    'getByEnglishTitle',
    'getChecklist',
    'post',
    'delete',
  ]);

  const userServiceSpy = jasmine.createSpyObj('userService', [
    'getUserId',
    'getAllUsers',
    'getUserById',
    'postUser',
  ]);

  const userId = 'someUserId';

  const rentalObjectTestPost: RentalObject = {
    englishTitle: 'Borrenose',
    title: 'Børrenøse',
    bedrooms: 3,
    beds: 4,
    sleeps: 6,
    description: 'Hyggelig hytte i Valdres',
    address: 'Børrenøsvegen 43, 2985 Tyinkrysset',
    petsAllowed: false,
    smokingAllowed: false,
    otherRules: [
      'Vær forsiktig med stearinslys.',
      'Vis hensyn overfor naboer mht. støy.',
      'Parkering på anvist sted.',
      'Nøkler oppbevares slik at de ikke kan identifiseres med hytta eller Communicate.',
    ],
    checkList: [
      'Testet røykvarslere.',
      'Rydded opp.',
      'Rengjort og fjernet matrester fra kjøleskap, stekeovn osv.',
      'Tømt søppel',
      '...',
    ],
    cardPicture: 'assets/pictures/borrenose/borrenose_4.jpeg',
    pictures: [
      'assets/pictures/borrenose/borrenose_1.jpeg',
      'assets/pictures/borrenose/borrenose_2.jpeg',
      'assets/pictures/borrenose/borrenose_3.jpeg',
    ],
    allocationPeriod: [
      new AllocationPeriod(
        new Date('2021-03-15'),
        new Date('2021-05-01'),
        new Date('2021-10-31')
      ),
      new AllocationPeriod(
        new Date('2021-09-15'),
        new Date('2021-11-01'),
        new Date('2021-04-30')
      ),
    ],
    applicationLimit: 2,
    periodPrices: [
      new PeriodPrice('Påsken', '1500kr per periode'),
      new PeriodPrice('En uke', '1500kr per uke'),
    ],
    disabledDays: [0, 1],
    rentalPeriods: [
      'Hyttebytte mellom to leiere foretas normalt på mandager. \nDen som avslutter leieforhold må være ute senest kl 12:00.' +
      '\nDen som starter leieforhold kan tidligst overta hytten kl 17:00.',
      'Påsken deles i 2 perioder: \nDel 1: Fredag før påske kl 17:00 – onsdag før påske kl 12:00' +
      '\nDel 2: Onsdag før påske kl 17:00 – mandag 2. påskedag kl 12:00.',
      'I Pinsen foretas hyttebytte mandag 2. pinsedag.'+
      '\nLeieperioden for pinsen er fra mandag før pinse kl 17:00 til mandag 2. pinsedag kl 12:00.',
      'Julen: Perioden kunngjøres på e-post.',
    ],
    latitude: 61.187065,
    longtitude: 8.213971,
    fireAlarm: ['en brannalarm', 'to brannalarm'],
    fireResponsible: 'Svein Veland',
    fireTestResponsible: 'Hver enkelt',
    fireDepartmentNumber: '110',
    fireAlarmNoFireInstructions:
      'Luft, vift med avis el.l for å fjerne røyken, eller trykk test/pauseknappen (midt på flaten) på den røykvarsleren som ' +
      'utløste alarmen. Hvis du trenger lengre tid enn 10 minutter på å få ut røyken, trykker du test/pauseknappen en gang' +
      'til når den begynner å ule igjen. Hvis røyk ikke er årsaken til falsk alarm, støvsug varslerne utvendig langs åpningen ' +
      'inn mot elektronikk og deteksjonskammer. Ikke åpne dekslet for å prøve rengjøre inne i varsleren. Ikke ta ut batteriene!',
    fireEquipment:
      '6 stk seriekoblede røykvarslere Modell Housegard Origo SA422WS',
    equipment:
      'Glass/kopper:\n' + 'Små kjøkkenglass 11 stk\n' + 'Vinglass 15 stk\n\n',
    directions: 'This section only shows up if it exists and is not empty.',
    otherPractialInformation:
      'Annen praktisk informasjon her. Feks. billeie, golf, skibakke osv.',
  };

  const bookingTestPostArray: Booking[] = [
    {
      id: '1',
      userId: '2',
      userName: 'someName',
      englishTitle: 'Borrenose',
      title: 'Børrenøse',
      startDate: new Date('2021-03-18'),
      endDate: new Date('2021-04-03'),
      isConfirmed: true,
    },
  ];

  const allocationServiceTestPeriod: AllocationPeriod = new AllocationPeriod(
    new Date('2021-03-15'),
    new Date('2021-05-01'),
    new Date('2021-10-31')
  );

  beforeEach(() => {
    TestBed.configureTestingModule({
      declarations: [RentalObjectComponent],
      schemas: [NO_ERRORS_SCHEMA],
      imports: [
        ModalModule.forRoot(),
        RouterTestingModule,
        HttpClientTestingModule,
      ],
      providers: [
        { provide: AllocationService, useValue: allocationServiceSpy },
        { provide: BookingService, useValue: bookingServiceSpy },
        { provide: UserService, useValue: rentalObjectServiceSpy },
        { provide: RentalObjectService, useValue: userServiceSpy },
        BsModalService,
        {
          // eslint-disable-next-line @typescript-eslint/naming-convention
          ActivatedRoute,
          useValue: {
            snapshot: {
              paramMap: {
                get : () => 'Borrenose',
              },
            },
          },
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(RentalObjectComponent);
    component = fixture.componentInstance;
    component.rentalObject = rentalObjectTestPost;
    fixture.detectChanges();

    allocationServiceMock = TestBed.inject(HttpTestingController);
    bookingServiceMock = TestBed.inject(HttpTestingController);
    rentalObjectServiceMock = TestBed.inject(HttpTestingController);
    userServiceMock = TestBed.inject(HttpTestingController);
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });
});
