using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Host;
using Microsoft.Extensions.Logging;
using System.Threading.Tasks;
using Microsoft.Extensions.Configuration;

namespace communicate_rental_function_app_DateTriggers
{
    public static class Scheduler
    {
        public static HttpClient httpClient = new HttpClient();

        [FunctionName("Scheduler")]
        public static async Task Run([TimerTrigger("0 0 6 * * *")]TimerInfo myTimer, ILogger log,
        Microsoft.Azure.WebJobs.ExecutionContext context)
        {
            var config = new ConfigurationBuilder()
            .SetBasePath(context.FunctionAppDirectory)
            .AddJsonFile("local.settings.json", optional: true, reloadOnChange: true)
            .AddJsonFile("settings.json", optional: true, reloadOnChange: true)
            .AddEnvironmentVariables()
            .Build();

            var ApiUrl = config.GetValue<string>("ApiUrl");

            string endpoint = ApiUrl + "FunctionApp/Trigger";

            var requestMessage = new HttpRequestMessage(HttpMethod.Post, endpoint);
            requestMessage.Headers.Add("Ocp-Apim-Subscription-Key", config.GetValue<string>("Apim-Key"));

            try
            {
                var response = await httpClient.SendAsync(requestMessage);

                response.EnsureSuccessStatusCode();
                log.LogInformation($"C# Timer trigger function executed at: {DateTime.Now}");
            }
            catch (System.Exception e)
            {
                log.LogInformation(e.ToString());
                log.LogInformation($"C# Timer trigger function could not execute at: {DateTime.Now}");
                throw;
            }
        }
    }
}

